@@ -40,6 +40,17 @@
     #include <idna.h>
 #endif
 
+#ifdef HAVE_HILDON_FM
+    #include <hildon/hildon-file-chooser-dialog.h>
+#endif
+
+#if HAVE_HILDON
+    #include <libosso.h>
+    #include <hildon/hildon.h>
+    #include <hildon-mime.h>
+    #include <hildon-uri.h>
+#endif
+
 static gchar*
 sokoke_js_string_utf8 (JSStringRef js_string)
 {
@@ -86,17 +97,106 @@ sokoke_js_script_eval (JSContextRef js_context,
     return value;
 }
 
-static void
-error_dialog (const gchar* short_message,
-              const gchar* detailed_message)
+void
+sokoke_message_dialog (GtkMessageType message_type,
+                       const gchar*   short_message,
+                       const gchar*   detailed_message)
 {
     GtkWidget* dialog = gtk_message_dialog_new (
-        NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "%s", short_message);
+        NULL, 0, message_type,
+        #if HAVE_HILDON
+        GTK_BUTTONS_NONE,
+        #else
+        GTK_BUTTONS_OK,
+        #endif
+        "%s", short_message);
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
                                               "%s", detailed_message);
-    gtk_widget_show (dialog);
     g_signal_connect_swapped (dialog, "response",
                               G_CALLBACK (gtk_widget_destroy), dialog);
+    gtk_widget_show (dialog);
+}
+
+/**
+ * sokoke_show_uri_with_mime_type:
+ * @screen: a #GdkScreen, or %NULL
+ * @uri: the URI to show
+ * @mime_type: a MIME type
+ * @timestamp: the timestamp of the event
+ * @error: the location of a #GError, or %NULL
+ *
+ * Shows the specified URI with an appropriate application,
+ * as though it had the specified MIME type.
+ *
+ * On Maemo, hildon_mime_open_file_with_mime_type() is used.
+ *
+ * See also: sokoke_show_uri().
+ *
+ * Return value: %TRUE on success, %FALSE if an error occurred
+ **/
+gboolean
+sokoke_show_uri_with_mime_type (GdkScreen*   screen,
+                                const gchar* uri,
+                                const gchar* mime_type,
+                                guint32      timestamp,
+                                GError**     error)
+{
+    gboolean success;
+    #if HAVE_HILDON
+    osso_context_t* osso;
+    DBusConnection* dbus;
+
+    osso = osso_initialize (PACKAGE_NAME, PACKAGE_VERSION, FALSE, NULL);
+    if (!osso)
+    {
+        g_print ("Failed to initialize libosso\n");
+        return FALSE;
+    }
+
+    dbus = (DBusConnection *) osso_get_dbus_connection (osso);
+    if (!dbus)
+    {
+        osso_deinitialize (osso);
+        g_print ("Failed to get dbus connection from osso context\n");
+        return FALSE;
+    }
+
+    success = (hildon_mime_open_file_with_mime_type (dbus,
+               uri, mime_type) == 1);
+    osso_deinitialize (osso);
+    #else
+    GFile* file = g_file_new_for_uri (uri);
+    gchar* content_type;
+    GAppInfo* app_info;
+    GList* files;
+    gpointer context;
+
+    #if GLIB_CHECK_VERSION (2, 18, 0)
+    content_type = g_content_type_from_mime_type (mime_type);
+    #else
+    content_type = g_strdup (mime_type);
+    #endif
+
+    app_info = g_app_info_get_default_for_type (content_type,
+        !g_str_has_prefix (uri, "file://"));
+    g_free (content_type);
+    files = g_list_prepend (NULL, file);
+    #if GTK_CHECK_VERSION (2, 14, 0)
+    context = gdk_app_launch_context_new ();
+    gdk_app_launch_context_set_screen (context, screen);
+    gdk_app_launch_context_set_timestamp (context, timestamp);
+    #else
+    context = g_app_launch_context_new ();
+    #endif
+
+    success = g_app_info_launch (app_info, files, context, error);
+
+    g_object_unref (app_info);
+    g_list_free (files);
+    g_object_unref (file);
+    #endif
+
+    return success;
 }
 
 /**
@@ -110,6 +210,8 @@ error_dialog (const gchar* short_message,
  * supports xdg-open, exo-open and gnome-open as fallbacks if
  * GIO doesn't do the trick.
  *
+ * On Maemo, hildon_uri_open() is used.
+ *
  * Return value: %TRUE on success, %FALSE if an error occurred
  **/
 gboolean
@@ -118,6 +220,11 @@ sokoke_show_uri (GdkScreen*   screen,
                  guint32      timestamp,
                  GError**     error)
 {
+    #if HAVE_HILDON
+    HildonURIAction* action = hildon_uri_get_default_action_by_uri (uri, NULL);
+    return hildon_uri_open (uri, action, error);
+    #else
+
     const gchar* fallbacks [] = { "xdg-open", "exo-open", "gnome-open" };
     gsize i;
 
@@ -140,39 +247,113 @@ sokoke_show_uri (GdkScreen*   screen,
     }
 
     return FALSE;
+    #endif
 }
 
 gboolean
 sokoke_spawn_program (const gchar* command,
                       const gchar* argument,
-                      gboolean     quote)
+                      gboolean     filename)
 {
-    GAppInfo* info;
-    GFile* file;
-    GList* files;
     GError* error;
 
     g_return_val_if_fail (command != NULL, FALSE);
     g_return_val_if_fail (argument != NULL, FALSE);
-    /* quote is ignored */
 
-    info = g_app_info_create_from_commandline (command,
-        NULL, G_APP_INFO_CREATE_NONE, NULL);
-    file = g_file_new_for_commandline_arg (argument);
-    files = g_list_append (NULL, file);
-
-    error = NULL;
-    if (!g_app_info_launch (info, files, NULL, &error))
+    if (filename)
     {
-        error_dialog (_("Could not run external program."), error->message);
-        g_error_free (error);
+        gboolean success;
+
+        #if HAVE_HILDON
+        osso_context_t* osso;
+        DBusConnection* dbus;
+
+        osso = osso_initialize (PACKAGE_NAME, PACKAGE_VERSION, FALSE, NULL);
+        if (!osso)
+        {
+            sokoke_message_dialog (GTK_MESSAGE_ERROR,
+                                   _("Could not run external program."),
+                                   "Failed to initialize libosso");
+            return FALSE;
+        }
+
+        dbus = (DBusConnection *) osso_get_dbus_connection (osso);
+        if (!dbus)
+        {
+            osso_deinitialize (osso);
+            sokoke_message_dialog (GTK_MESSAGE_ERROR,
+                                   _("Could not run external program."),
+                                   "Failed to get dbus connection from osso context");
+            return FALSE;
+        }
+
+        error = NULL;
+        /* FIXME: This is not correct, find a proper way to do this */
+        success = (osso_application_top (osso, command, argument) == OSSO_OK);
+        osso_deinitialize (osso);
+        #else
+        GAppInfo* info;
+        GFile* file;
+        GList* files;
+
+        info = g_app_info_create_from_commandline (command,
+            NULL, G_APP_INFO_CREATE_NONE, NULL);
+        file = g_file_new_for_commandline_arg (argument);
+        files = g_list_append (NULL, file);
+
+        error = NULL;
+        success = g_app_info_launch (info, files, NULL, &error);
         g_object_unref (file);
         g_list_free (files);
-        return FALSE;
+        #endif
+
+        if (!success)
+        {
+            sokoke_message_dialog (GTK_MESSAGE_ERROR,
+                _("Could not run external program."),
+                error ? error->message : "");
+            if (error)
+                g_error_free (error);
+            return FALSE;
+        }
+    }
+    else
+    {
+        /* FIXME: Implement Hildon specific version */
+        gchar* command_ready;
+        gchar** argv;
+
+        if (strstr (command, "%s"))
+            command_ready = g_strdup_printf (command, argument);
+        else
+            command_ready = g_strconcat (command, " ", argument, NULL);
+
+        error = NULL;
+        if (!g_shell_parse_argv (command_ready, NULL, &argv, &error))
+        {
+            sokoke_message_dialog (GTK_MESSAGE_ERROR,
+                                   _("Could not run external program."),
+                                   error->message);
+            g_error_free (error);
+            g_free (command_ready);
+            return FALSE;
+        }
+        g_free (command_ready);
+
+        error = NULL;
+        if (!g_spawn_async (NULL, argv, NULL,
+            (GSpawnFlags)G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
+            NULL, NULL, NULL, &error))
+        {
+            sokoke_message_dialog (GTK_MESSAGE_ERROR,
+                                   _("Could not run external program."),
+                                   error->message);
+            g_error_free (error);
+        }
+
+        g_strfreev (argv);
     }
 
-    g_object_unref (file);
-    g_list_free (files);
     return TRUE;
 }
 
@@ -356,6 +537,8 @@ sokoke_magic_uri (const gchar* uri,
     /* Just return if it's a javascript: or mailto: uri */
     if (g_str_has_prefix (uri, "javascript:")
      || g_str_has_prefix (uri, "mailto:")
+     || g_str_has_prefix (uri, "tel:")
+     || g_str_has_prefix (uri, "callto:")
      || g_str_has_prefix (uri, "data:"))
         return g_strdup (uri);
     /* Add file:// if we have a local path */
@@ -419,11 +602,22 @@ sokoke_format_uri_for_display (const gchar* uri)
 {
     if (uri && g_str_has_prefix (uri, "http://"))
     {
-        gchar* unescaped = g_uri_unescape_string (uri, NULL);
+        gchar* unescaped = g_uri_unescape_string (uri, " +");
         #ifdef HAVE_LIBSOUP_2_27_90
         gchar* path;
-        gchar* hostname = sokoke_hostname_from_uri (unescaped, &path);
-        gchar* decoded = g_hostname_to_unicode (hostname);
+        gchar* hostname;
+        gchar* decoded;
+
+        if (!unescaped)
+            return g_strdup (uri);
+        else if (!g_utf8_validate (unescaped, -1, NULL))
+        {
+            g_free (unescaped);
+            return g_strdup (uri);
+        }
+
+        hostname = sokoke_hostname_from_uri (unescaped, &path);
+        decoded = g_hostname_to_unicode (hostname);
 
         if (decoded)
         {
@@ -437,6 +631,15 @@ sokoke_format_uri_for_display (const gchar* uri)
         return unescaped;
         #elif HAVE_LIBIDN
         gchar* decoded;
+
+        if (!unescaped)
+            return g_strdup (uri);
+        else if (!g_utf8_validate (unescaped, -1, NULL))
+        {
+            g_free (unescaped);
+            return g_strdup (uri);
+        }
+
         if (!idna_to_unicode_8z8z (unescaped, &decoded, 0) == IDNA_SUCCESS)
             return unescaped;
         g_free (unescaped);
@@ -481,15 +684,6 @@ sokoke_container_show_children (GtkContainer* container)
     gtk_container_foreach (container, (GtkCallback)(gtk_widget_show_all), NULL);
 }
 
-void
-sokoke_widget_popup (GtkWidget*      widget,
-                     GtkMenu*        menu,
-                     GdkEventButton* event,
-                     SokokeMenuPos   pos)
-{
-    katze_widget_popup (widget, menu, event, (KatzeMenuPos)pos);
-}
-
 typedef enum
 {
     SOKOKE_DESKTOP_UNTESTED,
@@ -861,20 +1055,6 @@ sokoke_action_create_popup_menu_item (GtkAction* action)
     return menuitem;
 }
 
-/**
- * sokoke_image_menu_item_new_ellipsized:
- * @label: the text of the menu item
- *
- * Creates a new #GtkImageMenuItem containing an ellipsized label.
- *
- * Return value: a new #GtkImageMenuItem
- **/
-GtkWidget*
-sokoke_image_menu_item_new_ellipsized (const gchar* label)
-{
-    return katze_image_menu_item_new_ellipsized (label);
-}
-
 /**
  * sokoke_time_t_to_julian:
  * @timestamp: a time_t timestamp value
@@ -1140,6 +1320,7 @@ sokoke_find_data_filename (const gchar* filename)
     return g_build_filename (MDATADIR, filename, NULL);
 }
 
+#if !WEBKIT_CHECK_VERSION (1, 1, 14)
 static void
 res_server_handler_cb (SoupServer*        res_server,
                        SoupMessage*       msg,
@@ -1232,6 +1413,7 @@ sokoke_get_res_server (void)
 
     return res_server;
 }
+#endif
 
 gchar*
 sokoke_replace_variables (const gchar* template,
@@ -1295,3 +1477,37 @@ sokoke_window_activate_key (GtkWindow*   window,
 
     return FALSE;
 }
+
+/**
+ * sokoke_file_chooser_dialog_new:
+ * @title: a window title, or %NULL
+ * @window: a parent #GtkWindow, or %NULL
+ * @action: a #GtkFileChooserAction
+ *
+ * Creates a new file chooser dialog, as appropriate for
+ * the platform, with buttons according to the @action.
+ *
+ * The positive response is %GTK_RESPONSE_OK.
+ *
+ * Return value: a new #GtkFileChooser
+ **/
+GtkWidget*
+sokoke_file_chooser_dialog_new (const gchar*         title,
+                                GtkWindow*           window,
+                                GtkFileChooserAction action)
+{
+    const gchar* stock_id = GTK_STOCK_OPEN;
+    GtkWidget* dialog;
+
+    if (action == GTK_FILE_CHOOSER_ACTION_SAVE)
+        stock_id = GTK_STOCK_SAVE;
+    #ifdef HAVE_HILDON_FM
+    dialog = hildon_file_chooser_dialog_new (window, action);
+    #else
+    dialog = gtk_file_chooser_dialog_new (title, window, action,
+        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+        stock_id, GTK_RESPONSE_OK, NULL);
+    gtk_window_set_icon_name (GTK_WINDOW (dialog), stock_id);
+    #endif
+    return dialog;
+}
